#!/usr/bin/python3
#
#    DevDetect - small utility to detect new devices on local network
#    Copyright (C) 2017 CZ.NIC, z.s.p.o. (http://www.nic.cz/)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
import socket
import sys
import json
import time
import logging
from dev_detect.mac_addr_store import MacAddrStore

logger = logging.getLogger('devdetect')
logger.setLevel(logging.DEBUG)
handler = logging.StreamHandler()
handler.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

PAKOND_SOCKET='/tmp/pakon/socket'

class PakondReader():
	"""basically line reader, takes care of splitting stream from pakond into single json messages
	also takes cares of connecting/reconnecting
	"""
	def __init__(self, socket_path):
		self.socket_path = socket_path
		self.__get_connection()

	def __get_connection(self):
		"""get connection to pakond (keep trying forever, wait 5 second before each next attempt"""
		self.__buffer = b''
		while True:
			try:
				self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
				self.socket.connect(self.socket_path)
				logger.debug("connected to pakond")
				return
			except OSError:
				logger.debug("can't connect to pakond")
				time.sleep(5)

	def __reconnect(self):
		"""close connection, connect again"""
		logger.debug("reconnecting to pakond")
		self.socket.close()
		self.__get_connection()

	def read_line(self):
		"""keeps reading (blocking) until it gets (at least one) complete line, then it returns it"""
		while b'\n' not in self.__buffer:
			try:
				data = self.socket.recv(2048)
				if data:
					self.__buffer += data
				else:
					logger.debug("connection to pakond closed")
					self.__reconnect()
			except InterruptedError: #EINTR
				pass
			except OSError:
				logger.debug("connection to pakond lost")
				self.__reconnect()
		(line, self.__buffer) = self.__buffer.split(b'\n', 1)
		return line.decode('utf-8', 'ignore')

def main():
	reader = PakondReader(PAKOND_SOCKET)
	mac_store = MacAddrStore()
	while True:
		try:
			raw_data = reader.read_line()
			data = json.loads(raw_data)
			if data["method"] == "pakon.flow-update":
				mac_store.update(data["params"]["local"]["mac"], data["params"]["flags"]["src"])
			elif data["method"] == "pakon.proto-version":
				if data["params"]["version"] != 1:
					logger.error("Incompatible version of pakond protocol")
					sys.exit(1)
		except ValueError:
			logger.warn("received malformed record (invalid JSON) from pakond: {}".format(raw_data))
		except KeyError as err:
			if err.args[0] != "mac":
				logger.warn("received malformed report (missing expected key) from pakond: {}".format(raw_data))

if __name__ == "__main__":
	main()
